@@ -1,9 +1,11 @@
-#' Intraclass Correlation Coefficient (ICC)
+#' @title Intraclass Correlation Coefficient (ICC)
+#' @name icc
 #'
+#' @description
 #' This function calculates the intraclass-correlation coefficient (ICC) -
 #' sometimes also called *variance partition coefficient* (VPC) or
 #' *repeatability* - for mixed effects models. The ICC can be calculated for all
-#' models supported by `insight::get_variance()`. For models fitted with the
+#' models supported by [`insight::get_variance()`]. For models fitted with the
 #' **brms**-package, `icc()` might fail due to the large variety of
 #' models and families supported by the **brms**-package. In such cases, an
 #' alternative to the ICC is the `variance_decomposition()`, which is based
@@ -11,31 +13,36 @@
 #'
 #' @param model A (Bayesian) mixed effects model.
 #' @param re_formula Formula containing group-level effects to be considered in
-#'   the prediction. If `NULL` (default), include all group-level effects.
-#'   Else, for instance for nested models, name a specific group-level effect
-#'   to calculate the variance decomposition for this group-level. See 'Details'
-#'   and `?brms::posterior_predict`.
-#' @param ci Confidence resp. credible interval level. For `icc()` and `r2()`,
-#'   confidence intervals are based on bootstrapped samples from the ICC resp.
-#'   R2 value. See `iterations`.
-#' @param by_group Logical, if `TRUE`, `icc()` returns the variance
-#'   components for each random-effects level (if there are multiple levels).
-#'   See 'Details'.
+#' the prediction. If `NULL` (default), include all group-level effects.
+#' Else, for instance for nested models, name a specific group-level effect
+#' to calculate the variance decomposition for this group-level. See 'Details'
+#' and `?brms::posterior_predict`.
+#' @param ci Confidence resp. credible interval level. For `icc()`, `r2()`, and
+#' `rmse()`, confidence intervals are based on bootstrapped samples from the
+#' ICC, R2 or RMSE value. See `iterations`.
+#' @param by_group Logical, if `TRUE`, `icc()` returns the variance components
+#' for each random-effects level (if there are multiple levels). See 'Details'.
 #' @param iterations Number of bootstrap-replicates when computing confidence
-#'   intervals for the ICC or R2.
+#' intervals for the ICC, R2, RMSE etc.
 #' @param ci_method Character string, indicating the bootstrap-method. Should
-#'   be `NULL` (default), in which case `lme4::bootMer()` is used for
-#'   bootstrapped confidence intervals. However, if bootstrapped intervals cannot
-#'   be calculated this was, try `ci_method = "boot"`, which falls back to
-#'   `boot::boot()`. This may successfully return bootstrapped confidence intervals,
-#'   but bootstrapped samples may not be appropriate for the multilevel structure
-#'   of the model. There is also an option `ci_method = "analytical"`, which tries
-#'   to calculate analytical confidence assuming a chi-squared distribution.
-#'   However, these intervals are rather inaccurate and often too narrow. It is
-#'   recommended to calculate bootstrapped confidence intervals for mixed models.
+#' be `NULL` (default), in which case `lme4::bootMer()` is used for bootstrapped
+#' confidence intervals. However, if bootstrapped intervals cannot be calculated
+#' this way, try `ci_method = "boot"`, which falls back to `boot::boot()`. This
+#' may successfully return bootstrapped confidence intervals, but bootstrapped
+#' samples may not be appropriate for the multilevel structure of the model.
+#' There is also an option `ci_method = "analytical"`, which tries to calculate
+#' analytical confidence assuming a chi-squared distribution. However, these
+#' intervals are rather inaccurate and often too narrow. It is recommended to
+#' calculate bootstrapped confidence intervals for mixed models.
 #' @param verbose Toggle warnings and messages.
+#' @param null_model Optional, a null model to compute the random effect variances,
+#' which is passed to [`insight::get_variance()`]. Usually only required if
+#' calculation of r-squared or ICC fails when `null_model` is not specified. If
+#' calculating the null model takes longer and you already have fit the null
+#' model, you can pass it here, too, to speed up the process.
 #' @param ... Arguments passed down to `lme4::bootMer()` or `boot::boot()`
-#'   for bootstrapped ICC or R2.
+#' for bootstrapped ICC, R2, RMSE etc.; for `variance_decomposition()`,
+#' arguments are passed down to `brms::posterior_predict()`.
 #'
 #' @inheritParams r2_bayes
 #' @inheritParams insight::get_variance
@@ -58,6 +65,8 @@
 #'    applications and data analysis methods (2nd ed). Thousand Oaks: Sage
 #'    Publications.
 #'
+#' @inheritSection r2_nakagawa Supported models and model families
+#'
 #' @details
 #' ## Interpretation
 #' The ICC can be interpreted as "the proportion of the variance explained by
@@ -169,6 +178,9 @@ icc <- function(model,
                 ci = NULL,
                 iterations = 100,
                 ci_method = NULL,
+                null_model = NULL,
+                approximation = "lognormal",
+                model_component = NULL,
                 verbose = TRUE,
                 ...) {
   # special handling for smicd::semLme()
@@ -197,7 +209,14 @@ icc <- function(model,
   }
 
   # calculate random effect variances
-  vars <- .compute_random_vars(model, tolerance)
+  vars <- .compute_random_vars(
+    model,
+    tolerance = tolerance,
+    null_model = null_model,
+    approximation = approximation,
+    model_component = model_component,
+    verbose = verbose
+  )
 
   # return if ICC couldn't be computed
   if (is.null(vars) || all(is.na(vars))) {
@@ -230,7 +249,12 @@ icc <- function(model,
     # iccs between groups
     # n_grps <- length(vars$var.intercept)
     # level_combinations <- utils::combn(1:n_grps, m = n_grps - 1, simplify = FALSE)
-    # icc_grp <- sapply(level_combinations, function(v) vars$var.intercept[v[1]] / (vars$var.intercept[v[1]] + vars$var.intercept[v[2]]))
+    # icc_grp <- sapply(
+    #   level_combinations,
+    #   function(v) {
+    #     vars$var.intercept[v[1]] / (vars$var.intercept[v[1]] + vars$var.intercept[v[2]])
+    #   }
+    # )
     #
     # out2 <- data.frame(
     #   Group1 = group_names[sapply(level_combinations, function(i) i[1])],
@@ -256,11 +280,11 @@ icc <- function(model,
       # this is experimental!
       if (identical(ci_method, "analytical")) {
         result <- .safe(.analytical_icc_ci(model, ci))
-        if (!is.null(result)) {
+        if (is.null(result)) {
+          icc_ci_adjusted <- icc_ci_unadjusted <- NA
+        } else {
           icc_ci_adjusted <- result$ICC_adjusted
           icc_ci_unadjusted <- result$ICC_unadjusted
-        } else {
-          icc_ci_adjusted <- icc_ci_unadjusted <- NA
         }
       } else {
         result <- .bootstrap_icc(model, iterations, tolerance, ci_method, ...)
@@ -304,8 +328,6 @@ icc <- function(model,
 
 
 
-#' @param ... Arguments passed down to `brms::posterior_predict()`.
-#' @inheritParams icc
 #' @rdname icc
 #' @export
 variance_decomposition <- function(model,
@@ -411,7 +433,7 @@ print.icc <- function(x, digits = 3, ...) {
   }
 
   # separate lines for multiple R2
-  out <- paste0(out, collapse = "\n")
+  out <- paste(out, collapse = "\n")
 
   cat(out)
   cat("\n")
@@ -530,12 +552,18 @@ print.icc_decomposed <- function(x, digits = 2, ...) {
                                  components = c("var.fixed", "var.random", "var.residual"),
                                  name_fun = "icc()",
                                  name_full = "ICC",
+                                 null_model = NULL,
+                                 model_component = NULL,
+                                 approximation = "lognormal",
                                  verbose = TRUE) {
   vars <- tryCatch(
     insight::get_variance(model,
       name_fun = name_fun,
       name_full = name_full,
       tolerance = tolerance,
+      null_model = null_model,
+      approximation = approximation,
+      model_component = model_component,
       verbose = verbose
     ),
     error = function(e) {
@@ -568,7 +596,11 @@ print.icc_decomposed <- function(x, digits = 2, ...) {
 .boot_icc_fun <- function(data, indices, model, tolerance) {
   d <- data[indices, ] # allows boot to select sample
   fit <- suppressWarnings(suppressMessages(stats::update(model, data = d)))
-  vars <- .compute_random_vars(fit, tolerance, verbose = FALSE)
+  vars <- .compute_random_vars(
+    fit,
+    tolerance,
+    verbose = isTRUE(getOption("easystats_errors", FALSE))
+  )
   if (is.null(vars) || all(is.na(vars))) {
     return(c(NA, NA))
   }
@@ -581,7 +613,11 @@ print.icc_decomposed <- function(x, digits = 2, ...) {
 
 # bootstrapping using "lme4::bootMer"
 .boot_icc_fun_lme4 <- function(model) {
-  vars <- .compute_random_vars(model, tolerance = 1e-05, verbose = FALSE)
+  vars <- .compute_random_vars(
+    model,
+    tolerance = 1e-10,
+    verbose = isTRUE(getOption("easystats_errors", FALSE))
+  )
   if (is.null(vars) || all(is.na(vars))) {
     return(c(NA, NA))
   }
@@ -662,10 +698,10 @@ print.icc_decomposed <- function(x, digits = 2, ...) {
   }
 
   model_rank <- tryCatch(
-    if (!is.null(model$rank)) {
-      model$rank - df_int
-    } else {
+    if (is.null(model$rank)) {
       insight::n_parameters(model) - df_int
+    } else {
+      model$rank - df_int
     },
     error = function(e) insight::n_parameters(model) - df_int
   )
